#********************************************************************************
# qrtconf: a replacement for etc-update (part of qrtools)                       *
#                                                                               *
# Author: Kai Peter (4sh) (kp@aldox.org), ©2007, ©2017-                         *
#                                                                               *
# Version: 0.31                                                                 *
# License: ISC license                                                          *
#********************************************************************************
# Configuration section (defaults)
#
# the top dirctory for variable data used by qrtools (read-only).
declare -r BCKDIR=/var/lib/qrtools
# default settings (don't edit, use config file instead)
INTERACTIVE="1"
VERBOSITY="1"
# get config from file (overwrites defaults)
[ -f "/etc/portage/qrtools/$0.conf" ] && . "/etc/portage/qrtools/$0.conf"

# the subfolder contains the latest version of a distribution config file
[ ! -d "$BCKDIR" ] && mkdir -p "$BCKDIR"

# functions section *************************************************************
# check if required tools are in $PATH
function getTools () {
  for T in diff patch
  do
	which "$T" 2>/dev/null 1>/dev/null
	EC=$?
	[ ! "$EC" = "0" ] && echo "Couldn't find $T - aborting!" && exit 1
  done
}

function chkDiff () {
  local EC=0
  [ "$VERBOSITY" -gt 1 ] && echo "Checking for differences ..."

  # check if the new ._cfg* file has changes against the backup'd one ...
  [ -f "$BCKFILE" ] && diff "$BCKFILE" "$NF" 2>&1>/dev/null && EC=$?
  # ... and if yes then create a patch (exitcode != 0)
  if [ "$EC" -eq "1" ] ; then
	[ "$VERBOSITY" -gt 0 ] && echo "Creating patch 1 ..."
	diff -Naur "$BCKFILE" "$NF" > "$BCKFILE.1.patch"
# TODO: doPatch from here ...
   else
	[ "$VERBOSITY" -gt 0 ] && echo "Nothing to do for patch 1."
  fi
  return
}

# create a backup file in '/var/lib/qrtconf'
function doBackup () {
  [ ! -d "$DSTFLDR" ] && mkdir -p "$DSTFLDR" && \
  if [ "$VERBOSITY" -gt 0 ] ; then
    printf "\033[1mCreated folder: $DSTFLDR!\n\033[0m" ; fi
  # 
  cp -p "$NF" "$DSTFLDR/$FN"
}

function doPatch () {
  local EC=0
  local OPTS="-f"				# some options for patch
  if [ "$VERBOSITY" -gt 1 ] ; then
    OPTS="$OPTS --verbose"
    printf "\033[1mPatching: $CFGFILE!\n\033[0m"
  fi

#TODO: check if cfg file in /etc is a link (e.g. procmailrc)

  # doing a loop gives more flexibility for future purposes - perhaps :-)
  for P in "1" "2"
  do
	if [ -f "$BCKFILE.$P.patch" ] ; then
	# first use '--dry-run' and check exit code, output depends on verbosity level
	[ "$VERBOSITY" -gt 0 ] && patch --dry-run "$OPTS" "$CFGFILE" < "$BCKFILE.$P.patch" \
	  || patch --dry-run "$OPTS" "$CFGFILE" < "$BCKFILE.$P.patch" 2>&1>/dev/null
	EC=$?
	  if [ "$EC" = "0" ] ; then		# if '--dry-run' looks good, do it ...
		# ... and like above, suppress output depending of verbosity level
		[ "$VERBOSITY" -gt 0 ] && patch "$OPTS" "$CFGFILE" < "$BCKFILE.$P.patch" \
		  || patch "$OPTS" "$CFGFILE" < "$BCKFILE.$P.patch" 2>&1>/dev/null
		EC=$?
	  fi
	  rm -f "$BCKFILE.$P.patch"		# remove temporary files
	fi
  done
  [ "$VERBOSITY" -gt 1 ] && echo "Replacing backup file ..."
  [ "$EC" = "0" ] && mv "$NF" "$DSTFLDR/$FN"	# last '$EC' is from 'patch' cmd
}

function showHelp () {
  echo -e "Usage: qrtconf [option [param]]"
  echo -e ""
  echo -e "Options:"
  echo -e "   -1 \t process first file only and exit"
  echo -e "   -h \t show this help and exit"
  echo -e "   -i \t interactive mode (default)"
  echo -e "   -v \t be more verbose"
  echo -e ""
  exit 0
}
#********************************************************************************
# Start processing
while getopts "1ahiv::" O; do
  case "${O}" in
    1) ONE=1;;
    a) INTERACTIVE=0;;		# switch interactive mode off
#    b) ;;
    h) showHelp ;;
    i) INTERACTIVE=1;;		# interactive mode (default)
    v) VERBOSITY=${OPTARG}  # valid are 0, 1, and 2
  	   [ "$VERBOSITY" ] || VERBOSITY="1";;
  	*) echo "Invalid option!" ; showHelp;;
  esac
done ; shift $((OPTIND-1))

# check for valid variables, fallback to default settings
[ "$INTERACTIVE" ] && [ "$INTERACTIVE" = "0" ] || INTERACTIVE=1
[ "$INTERACTIVE" = "1" ] && [ "$VERBOSITY" = "0" ] && VERBOSITY=1

getTools			# check for required system tools

# get all "new" config files in /etc and subfolders
# Variables:
# $NF    	- 'new' files prefixed with "._cfg????_*"
# $DN    	- directory ("/etc/*"), location, the dirname of $NF
# $FN		- file name without the "._cfg????_" prefix
NbrOfFiles=0	# count the number of processed files
for NF in $(ls -ld $(find /etc) | grep '._cfg' | awk '{ print $9 }') ; do
  DN=$(dirname $NF)               # get the location of prefixed config file
  FN=$(echo $NF | cut -d_ -f3-)   # get the config file name w/o path and prefix

  # first let's use some variable to consolidate
  DSTFLDR="$BCKDIR$DN"		# destination folder (beware $DN starts with a '/')
  BCKFILE="$DSTFLDR/$FN"	# the backup'ed file w/ full path (in /var/...)
  CFGFILE=$DN/$FN			# the "real" config file

  if [ -f "$BCKFILE" ] ; then		# check if a backup'd file exists ...
	[ "$VERBOSITY" -gt "0" ] && echo "Corresponding backup found!"
   else  # ... or lets do a backup simply
	if [ "$INTERACTIVE" = "1" ] ; then
	  # get interactive result (and we can use bash :-))
	  PROMPT="Do you want to create a backup? (yes/No): "
	  read -n 1 -p "$PROMPT" AW
	  echo							# do a line break after read
	  [ "$AW" = "Y" ] && AW="y"		# convert upper case letter
	  [ ! "$AW" ] && AW="n"			# invalid input defaults to NO
	  [ $AW = "y" ] && doBackup    	# 'echo': do a linebreak on NO
	 else
	  doBackup	# (see http://gentoo.dyndn.es/qrtconf#practice for more info)
	fi
  fi
  chkDiff		# calling the functions 'chkDiff' and 'doPatch' always is safe,
  doPatch		# because if there is nothing to do they will do nothing
  NbrOfFiles=$((NbrOfFiles+1))
  [ "$ONE" ] && break		# respect option '-1'
done
printf "qrtconf: $NbrOfFiles files processed!\n"
exit 0
